@@ -16,6 +16,8 @@
 #include <mapnik/load_map.hpp>
 #include <mapnik/image_reader.hpp>
 #include <mapnik/image_util.hpp>
+#include <mapnik/feature_factory.hpp>
+#include <mapnik/raster.hpp>
 
 #include <sstream>
 #include <fstream>
@@ -246,3 +248,129 @@ TEST_CASE( "raster tile output 2", "should be able to overzoom raster" ) {
         mapnik::save_to_file(im,"test/fixtures/actual-3.png","png32");
     }
 }
+
+TEST_CASE( "raster tile output 3", "should be able to round trip image with alpha" ) {
+    mapnik::box2d<double> bbox(-20037508.342789,-20037508.342789,20037508.342789,20037508.342789);
+
+    // create a vtile from scratch with a raster
+
+    vector_tile::Tile tile;
+    std::string format("png32");
+    {
+        mapnik::vector_tile_impl::backend_pbf backend(tile,16);
+        backend.start_tile_layer("layer");
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(std::make_shared<mapnik::context_type>(),1));
+        backend.start_tile_feature(*feature);
+        std::unique_ptr<mapnik::image_reader> reader(mapnik::get_image_reader("./test/fixtures/alpha-white-2.png"));
+        if (!reader.get()) {
+            throw std::runtime_error("could not open image");
+        }
+        mapnik::image_rgba8 im_data(reader->width(),reader->height());
+        reader->read(0,0,im_data);
+        backend.add_tile_feature_raster(mapnik::save_to_string(im_data,format));
+        backend.stop_tile_feature();
+        backend.stop_tile_layer();
+    }
+    {
+        REQUIRE(1 == tile.layers_size());
+        vector_tile::Tile_Layer const& layer = tile.layers(0);
+        REQUIRE(1 == layer.features_size());
+        vector_tile::Tile_Feature const& f = layer.features(0);
+        REQUIRE(f.has_raster());
+        std::string const& ras_buffer = f.raster();
+        REQUIRE(!ras_buffer.empty());
+        std::unique_ptr<mapnik::image_reader> reader(mapnik::get_image_reader(ras_buffer.data(),ras_buffer.size()));
+        if (!reader.get()) {
+            throw std::runtime_error("could not open image bytes");
+        }
+        mapnik::image_rgba8 im_data(reader->width(),reader->height());
+        reader->read(0,0,im_data);
+        unsigned diff = testing::compare_images(im_data,"./test/fixtures/alpha-white-2.png",0,true);
+        CHECK(0 == diff);
+        if (diff > 0) {
+            mapnik::save_to_file(im_data,"test/fixtures/actual-4.png",format);
+        }
+
+    }
+
+    // Now actually re-render to trigger the raster being passed through the processor
+    // and confirm raster still looks correct
+    {
+        // create datasource wrapping raster
+        unsigned tile_size = 256;
+        std::shared_ptr<mapnik::vector_tile_impl::tile_datasource> ds = std::make_shared<
+                                        mapnik::vector_tile_impl::tile_datasource>(
+                                            tile.layers(0),0,0,0,tile_size);
+        // before rendering let's query the raster directly to ensure
+        // the datasource returns it correctly.
+        mapnik::query q(bbox);
+        mapnik::featureset_ptr fs = ds->features(q);
+        mapnik::feature_ptr feat = fs->next();
+        mapnik::raster_ptr const& source = feat->get_raster();
+        CHECK(source->data_.is<mapnik::image_rgba8>());
+        mapnik::image_rgba8 const& source_data = mapnik::util::get<mapnik::image_rgba8>(source->data_);
+        unsigned diff = testing::compare_images(source_data,"./test/fixtures/alpha-white-2.png",0,true);
+        CHECK(0 == diff);
+        if (diff > 0) {
+            mapnik::save_to_file(source_data,"test/fixtures/actual-5.png",format);
+        }
+
+        // okay, now we'll re-render another vector tile from original
+        // which triggers the raster to be resampled
+        mapnik::Map map(tile_size,tile_size,"+init=epsg:3857");
+        mapnik::layer lyr("layer",map.srs());
+        ds->set_envelope(bbox);
+        lyr.set_datasource(ds);
+        map.add_layer(lyr);
+        map.zoom_to_box(bbox);
+
+        typedef mapnik::vector_tile_impl::backend_pbf backend_type;
+        vector_tile::Tile round_tipped_tile;
+        backend_type backend(round_tipped_tile,16);
+        typedef mapnik::vector_tile_impl::processor<backend_type> renderer_type;
+        mapnik::request m_req(tile_size,tile_size,bbox);
+        renderer_type ren(backend,
+                          map,
+                          m_req,
+                          1.0,
+                          0,
+                          0,
+                          1,
+                          false,
+                          format,
+                          mapnik::SCALING_BILINEAR);
+        ren.apply();
+        REQUIRE(1 == round_tipped_tile.layers_size());
+        vector_tile::Tile_Layer const& layer = round_tipped_tile.layers(0);
+        REQUIRE(1 == layer.features_size());
+        vector_tile::Tile_Feature const& f = layer.features(0);
+        REQUIRE(f.has_raster());
+        std::string const& ras_buffer = f.raster();
+        REQUIRE(!ras_buffer.empty());
+        std::unique_ptr<mapnik::image_reader> reader(mapnik::get_image_reader(ras_buffer.data(),ras_buffer.size()));
+        if (!reader.get()) {
+            throw std::runtime_error("could not open image bytes");
+        }
+
+        mapnik::image_rgba8 im_data(reader->width(),reader->height());
+        reader->read(0,0,im_data);
+
+        if (!mapnik::util::exists("test/fixtures/expected-4.png")) {
+            mapnik::save_to_file(im_data,"test/fixtures/expected-4.png","png32");
+        }
+
+        diff = testing::compare_images(im_data,"test/fixtures/expected-4.png",0,true);
+        CHECK(0 == diff);
+        if (diff > 0) {
+            mapnik::save_to_file(im_data,"test/fixtures/actual-7.png",format);
+        }
+
+        // visually identical but scaling causes some pixels shifts compared to original
+        diff = testing::compare_images(im_data,"./test/fixtures/alpha-white-2.png",0,true);
+        CHECK(13853 == diff);
+        if (diff > 0) {
+            mapnik::save_to_file(im_data,"test/fixtures/actual-7.png",format);
+        }
+    }
+
+}
